//
// FullcalendarTimeline - Constructor and initialisation
//

class FullcalendarTimeline {

  // Assumption:
  // Each timeline element id is unique <div id='timelineElementId'>, and associated form fields
  // are encapsulated by a unique id (e.g. <fieldset id='formFieldsId'>)
  // (TBD: Add to documentation)

  constructor(formFieldsId, timelineElementId, current_resource_model_group) {
    // Mechanism to pass (this) to the handlers
    var fullCalendarTimelineObject = this;

    this._formFieldsId = formFieldsId;
    this._formFieldsElement = $('#' + formFieldsId);
    this._formFieldsWrapper = this._formFieldsElement;

    this._timelineElementId = timelineElementId;
    this._timelineElement = $('#' + timelineElementId);
    this._timelineViewWrapper = this._timelineElement.closest('#timeline-view');

    // Get application preferences
    this._applicationPreferences = new ApplicationPreferences();

    this._currentResourceModelGroup = 
      ((typeof current_resource_model_group === 'undefined') || (current_resource_model_group === null)) ?
        null : current_resource_model_group;

    setCookie('current_resource_model_group', this._currentResourceModelGroup, 1);

    // Initialise information for EventUnits
    //
    // Add formId as a hidden field (cookie prefix)
    this._formFieldsElement.append("<input type='hidden' name='cookie_prefix' value='" + this._formFieldsId + "' />");
    this.updateEventUnitsFormFields();

    this._fullcalendarObject = new FullCalendar.Calendar(this._timelineElement[0], {
      schedulerLicenseKey: 'GPL-My-Project-Is-Open-Source',
      plugins: ['interaction', 'resourceTimeline'], //, 'rrule'],
      defaultDate: new Date().setMinutes(0), // now
      timeZone: 'local',
      editable: true, // enable draggable events
      aspectRatio: 1.8, // default: 1.35
      scrollTime: '00:00', // undo default 6am scrollTime
      height: 'auto', // If "auto", the view’s contents will assume a natural height and no scrollbars will be used (this one!)
      // height: '100%', // If "100%", the height of the calendar will match the height of its parent container element

      header: {
        left: 'promptCourse,promptTrainer,promptRoom today prev,next',
        center: 'title',
        right: 'zoomIn,zoomOut'
      },

      customButtons: {
        // zoomIn: {
        //   text: '<',
        // },
        // zoomOut: {
        //   text: '>',
        // },
        promptCourse: {
          text: '+Course',
          click: function () {
            var title = prompt('Course name');
            if (title) {
              $('#fullcalendar_scheduler').fullCalendar(
                'addResource',
                { title: title },
                true // scroll to the new resource?
              );
            }
          }
        },
        promptTrainer: {
          text: '+Trainer',
          click: function () {
            var title = prompt('Trainer name');
            if (title) {
              $('#fullcalendar_scheduler').fullCalendar(
                'addResource',
                { title: title },
                true // scroll to the new resource?
              );
            }
          }
        },
        promptRoom: {
          text: '+Room',
          click: function () {
            var title = prompt('Room name');
            if (title) {
              $('#fullcalendar_scheduler').fullCalendar(
                'addResource',
                { title: title },
                true // scroll to the new resource?
              );
            }
          }
        }
      },

      //defaultView: 'timeline3Days',
      //defaultView: 'timeline1Day',
      //defaultView: 'timeline',
      defaultView: 'resourceTimelineDay',

      // visibleRange: function (currentDate) {
      //   return {
      //     start: currentDate, //window.visibleRangeStartDate,
      //     end: currentDate.clone().add(3, 'days') //window.visibleRangeEndDate
      //   };
      // },

      // validRange: function(nowDate) {
      //   return {
      //     start: dateAdd(nowDate, -3, 'days'), //window.visibleRangeStartDate,
      //     end: dateAdd(nowDate, 3, 'days') //window.visibleRangeEndDate
      //   };
      // },

      // views: {
      //   resourceTimeline2Days: {
      //     type: 'resourceTimeline',
      //     duration: { days: 2 }
      //   },
      //   resourceTimeline3Days: {
      //     type: 'resourceTimeline',
      //     duration: { days: 3 }
      //   },
      //   resourceTimeline: {
      //     // options apply to resourceTimeline views (resourceTimelineDay, resourceTimelineWeek, resourceTimelineMonth)
      //     visibleRange: function (currentDate) {
      //       return {
      //         start: window.visibleRangeStartDate,
      //         end: window.visibleRangeEndDate
      //       };
      //     },
      //   },
      //   dayGrid: {
      //     // options apply to dayGridDay, dayGridWeek and dayGridMonth views
      //   },
      //   timeGrid: {
      //     // options apply to timeGridDay and timeGridWeek views
      //   },
      //   week: {
      //     // options apply to dayGridWeek and timeGridWeek views
      //   },
      //   day: {
      //     // options apply to dayGridDay and timeGridDay views
      //   }

      // },

      resourceAreaWidth: '20%',
      resourceGroupField: 'resourceType',

      // If refetchResourcesOnNavigate is set to true, the network request will
      // be made with additional parameters: the start/end date of the newly
      // visible window of time, as well as the calendar’s timezone
      //
      // example request: /resource_planner/resources.json?start=2017-12-01&end=2017-01-12&timezone=UTC

      refetchResourcesOnNavigate: true,
      resources: {
        url: '/resource_planner/resources.json',
        type: 'GET',
      },

      eventSources: getEventSources(),

      // Callbacks

      // Triggered while a resource is being rendered. A hook for modifying its DOM.
      resourceRender: function (renderInfo) {
        // Initialise resource quantity requirements inputs
        var resourceData = renderInfo.resource.id.split('-'); // e.g. "Equipment-1-150"
        var maxQuantity = resourceData.pop();
        var fieldId = resourceData.join('-');

        if (Number(maxQuantity) > 1) {
          var resourceQuantityInput = `<input type='number' id='${fieldId}' min='0' max='${maxQuantity}' maxlength='2' size='2' value='0' style='font-size: 75%;' />`;
          var resourceAvailabilitySpan = `<span id='${fieldId}' style='font-size: 75%; color: blue;'>${maxQuantity}</span>`;
          $(renderInfo.el).find('.fc-cell-text').append('<br>'+resourceQuantityInput+resourceAvailabilitySpan);
        }

        // Add handler to select/unselect the resource
        $(renderInfo.el).on('click', function (event) {
          fullCalendarTimelineObject.resourceSelectedHandler(renderInfo);
        });

        // Disable any event handlers for the 'resourceQuantityInput' field
        $(renderInfo.el).find('input').on({
          click: function (event) {
            event.stopPropagation();
          },
          change: function (event) {
            event.stopPropagation();
            fullCalendarTimelineObject.renderSelectedResources();
          },
          mousedown: function (event) {
            event.stopPropagation();
          }
        });

        // This would be better (only one handler for all inputs):
        // $(renderInfo.el).on({
        //   click: function (event) {
        //     event.stopPropagation();
        //   },
        //   change: function (event) {
        //     event.stopPropagation();
        //   },
        // mousedown: function (event) {
        //   event.stopPropagation();
        // }
        // }, 'input');

      },

      viewSkeletonRender: function (info) {
        // View’s non-date-related DOM structure has been rendered.
        // This callback will get triggered when the initial view renders or when
        // the user changes the view, but before the datesRender callback fires.
      },

      // All date/time cells have been rendered.
      // This triggers after viewSkeletonRender callback but before the eventRender callbacks.
      datesRender: function (info) {

        // fullCalendarTimelineObject.attachTimelineManipulationHandlers();
        // fullCalendarTimelineObject.renderTopScrollBar();
        fullCalendarTimelineObject.initialiseHighlighterbarTooltip();

        if (typeof fullCalendarTimelineObject._eventUnits !== 'undefined') {
          fullCalendarTimelineObject.renderEvents(fullCalendarTimelineObject._eventUnits, fullCalendarTimelineObject._startingDate, fullCalendarTimelineObject._endingDate);
        }

        // renderSelectedResources();
      },

      // Triggered while an event is being rendered. A hook for modifying its DOM.
      eventRender: function (info) {
        fullCalendarTimelineObject.renderTopScrollBar();
      },

      eventDrop: function (event, delta, revertFunc) {
        if (!confirm(event.title + " was changed to " + event.start.format() + "\nAre you sure about this change?")) {
          revertFunc();
        }
      },

    }); // end new FullCalendar.Calendar

  } // end constructor

  // Getters/Setters
  get fullcalendarObject() {
    return this._fullcalendarObject;
  }

  get formFieldsWrapper() {
    return this._formFieldsElement;
  }

  get timelineElement() {
    return this._timelineElement;
  }

  get timelineViewWrapper() {
    return this._timelineViewWrapper;
  }

  get fullcalendarTimelineTimelineManipulator() {
    return this._fullcalendarTimelineTimelineManipulator;
  }

  get fullcalendarTimelineResourceManipulator() {
    return this._fullcalendarTimelineResourceManipulator;
  }

  get fullcalendarTimelineHighlighterBarManipulator() {
    return this._fullcalendarTimelineHighlighterBarManipulator;
  }

  // Initialise and render
  render() {
    this._fullcalendarObject.render();

    this.initialiseI18nControls();
    this.initialiseTimelineManipulationControls();

    this.initialiseFormFieldAribitration();

    // During initialisation, it is best for the the form arbitration to be applied
    // before the highlighter bar is initialised, since the highlighter bar depends
    // on the '_formData' for initial positioning
    this.initialiseHighlighterBarManipulationControls();
    this.initialiseHighlighterbarTooltip();

    this.initialiseResourceManipulationControls();

    this.renderTopScrollBar();

    // Render the highlighter bar
    this.formUpdateHandler();
    this.validateFormData();
    this.recurenceInputStatus(this._formData.event_rrule);
  }

  renderTopScrollBar() {
    // if ($('#doubleScroll_padding').length == 0) {
    //   $('#fullcalendar_scheduler tbody.fc-body td.fc-resource-area div.fc-scroller').prepend($('<div id="doubleScroll_padding"></div>'));
    // }

    this._timelineElement.find('tbody.fc-body td.fc-resource-area div.fc-scroller').doubleScroll({
      onlyIfScroll: false,      // top scrollbar is not shown if the bottom one is not present
      resetOnWindowResize: true // recompute the top ScrollBar requirements when the window is resized
    });

    this._timelineElement.find('tbody.fc-body td.fc-time-area div.fc-scroller').doubleScroll({
      onlyIfScroll: true,       // top scrollbar is not shown if the bottom one is not present
      resetOnWindowResize: true // recompute the top ScrollBar requirements when the window is resized
    });
  }

} // end class
