//
// FullcalendarTimeline - Highlighter bar manipulation controls
//

// Initialise timeline manipulation controls
FullcalendarTimeline.prototype.initialiseFormFieldAribitration = function () {

  // Mechanism to pass (this) to the handlers
  var fullCalendarTimelineObject = this;

  this._formData = {};
  this._iCalData = {};

  // clear all fields
  // var selected_recurrence = this.getFormField('select#event_rrule').val();
  // this.clearFields(selected_recurrence);

  // Attach date/time selectors to the page
  $('.datetimepick').datetimepicker({
    format: 'Y-m-d H:i',
    step: 15,
  });

  $('.datepick').datetimepicker({
    timepicker: false,
    format: 'Y-m-d',
  });

  // Initialise timezone fields from browser timezone
  var browser_timezone = getCookie('browser.timezone');
  this.getFormField(formField().TZOFFSETFROM).children("select option[value='" + browser_timezone + "']").prop('selected', true);
  this.getFormField(formField().TZOFFSETTO).children("select option[value='" + browser_timezone + "']").prop('selected', true);
  this.getFormField('#resource_planner_event_until_timezone_timezone').children("select option[value='" + browser_timezone + "']").prop('selected', true);


  // Attach event handlers to all relevant form fields

  //
  // common fields
  //

  // title
  this.getFormField(formField().SUMMARY).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
  });

  // description
  this.getFormField(formField().DESCRIPTION).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
  });

  // all_day
  this.getFormField(formField().ALLDAY).on('change click', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.allDayHandler();
    //fullCalendarTimelineObject.manageStartDateOrDurationChange();
    fullCalendarTimelineObject.validateFormData();
  });

  // start_date
  this.getFormField(formField().DTSTART).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.manageStartDateOrDurationChange();
    fullCalendarTimelineObject.validateFormData();
  });

  this.getFormField(formField().TZOFFSETFROM).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.manageStartDateOrDurationChange();
    fullCalendarTimelineObject.validateFormData();
    fullCalendarTimelineObject.timeZoneConverter();
    fullCalendarTimelineObject.recurenceInputStatus(fullCalendarTimelineObject._formData.event_rrule);
  });

  // duration
  this.getFormField(formField().DURATION).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.manageStartDateOrDurationChange();
    fullCalendarTimelineObject.validateFormData();
  });

  this.getFormField(formField().DURATION_UNITS).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.manageDurationUnitsChange();
    fullCalendarTimelineObject.validateFormData();
  });

  // event_rrule (recurence rule select)
  this.getFormField('#event_rrule').on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.handleRecurrenceSelectorChange();
    fullCalendarTimelineObject.validateFormData();
  });

  //
  // no_repeat fields
  //

  // end_date
  this.getFormField(formField().DTEND).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.manageEndDateChange();
    fullCalendarTimelineObject.validateFormData();
  });

  this.getFormField(formField().TZOFFSETTO).on('change input', function () {
    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
    fullCalendarTimelineObject.timeZoneConverter();
    fullCalendarTimelineObject.recurenceInputStatus(fullCalendarTimelineObject._formData.event_rrule);
  });

  //
  // recurence_pattern fields
  //
  this.getFormField('.day_buttons label').on('click change input', function () {
    var checkbox = $(this).prev('input:checkbox:first');
    if (checkbox.is(':checked')) {
      // it is currently checked, un-ckeck it
      checkbox.prop('checked', false);
      $(this).removeClass('day-button-selected');
    } else {
      // it is currently un-checked, ckeck it
      checkbox.prop('checked', true);
      $(this).addClass('day-button-selected');
    }

    fullCalendarTimelineObject.validateFormData();
  });

  this.getFormField('.monthly_day_buttons label').on('click change input', function () {
    var checkbox = $(this).prev('input:checkbox:first');
    if (checkbox.is(':checked')) {
      // it is currently checked, un-ckeck it
      checkbox.prop('checked', false);
      $(this).removeClass('day-button-selected');
    } else {
      // it is currently un-checked, ckeck it
      checkbox.prop('checked', true);
      $(this).addClass('day-button-selected');
    }

    fullCalendarTimelineObject.validateFormData();
  });

  // Recurrence pattern - event input fields (every n units)
  this.getFormField('.recurring_event_data input').on('change input', function () {
    // check the adjacent checkbox
    $(this).parents('.row:first').find('input:first')[0].checked = true;

    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
  });

  // Recurrence pattern - event dropdown day select fields ( e.g. 'every Nth Monday')
  this.getFormField('.recurring_event_data select').on('change input', function () {
    // check the adjacent checkbox
    $(this).parents('.row:first').find('input:first')[0].checked = true;

    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
    fullCalendarTimelineObject.recurenceInputStatus($.trim(fullCalendarTimelineObject.getFormField('#event_rrule').val()).toLowerCase());
  });

  // Recurrence pattern - event day/date table fields (the day of the week/month)
  this.getFormField('.recurring_event_data label').on('click change input', function () {
    // check the adjacent checkbox
    $(this).parents('.row:first').find('input:first')[0].checked = true;

    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
  });

  //
  // range_of_recurence fields
  //
  this.getFormField('.range_of_recurence input').on('change input', function () {
    // check the adjacent checkbox
    $(this).parents('.row:first').find('input:first')[0].checked = true;

    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
  });

  this.getFormField('.range_of_recurence select').on('change input', function () {
    // check the adjacent checkbox
    $(this).parents('.row:first').find('input:first')[0].checked = true;

    fullCalendarTimelineObject.formUpdateHandler();
    fullCalendarTimelineObject.validateFormData();
  });

  // Submit button handler
  // this.getFormField('input[type=submit]').on('click', function () {
  //   //fullCalendarTimelineObject.submitButtonHandler();
  //   fullCalendarTimelineObject.validateFormData();
  // });

  // Initialise...
  this.formUpdateHandler();
  this.validateFormData();
}

//=============================================================================
//
// Helper methods for field changes
//
//=============================================================================

//=============================================================================
// If start_date, duration or duration_units is changed, validate and update
// the DTEND field
FullcalendarTimeline.prototype.manageStartDateOrDurationChange = function () {
  var startDate = this._formData.start_date_with_timezone;
  //var startDate = new Date(this._formData.start_date);
  var startDateIsValid = (startDate.toString() != 'Invalid Date');
  var endDate = new Date(this._formData.end_date);
  var endDateIsValid = (endDate.toString() != 'Invalid Date');
  //var duration = this._formData.duration;
  var duration = $.trim(this.getFormField(formField().DURATION).val());
  var durationUnits = this._formData.duration_units;
  //var all_day = this._formData.all_day;

  if (startDateIsValid) {
    if (duration == '') {
      this.getFormField(formField().DTEND).val('');
      this.getFormField(formField().ALLDAY)[0].checked = false;
      endDateIsValid = 'Invalid Date';
    }

    if (duration > 0 && !isNaN(duration)) {
      // Align DTEND with DTSTART/DURATION change
      endDate = dateAdd(startDate, duration, durationUnits);
      var endTZ = this._formData.resource_planner_event_dtend_timezone;
      moment.tz.setDefault(endTZ);
      endDate = moment(endDate).format();
      this.getFormField(formField().DTEND).val(moment(endDate).format("YYYY-MM-DD HH:mm"))
      this.getFormField(formField().ALLDAY)[0].checked = false;
    }

    if (!endDateIsValid) {
      startDate = new Date(startDate);
      endDate = new Date(endDate);
      if (endDate.getTime() <= startDate.getTime()) {
        // End Date must be >= start Date
        endDate = startDate;
        this.getFormField(formField().DURATION).val('');
        this.getFormField(formField().DTEND).val(formatDate(endDate));
      } else {
        // Update duration
        duration = endDate.diff(startDate, this._formData.duration_units);
        this.getFormField(formField().DURATION).val(duration);
      }
    }
  }
  //this.getFormField(formField().ALLDAY)[0].checked = false;
}

//=============================================================================
// If end_date changed, validate and update DURATION
FullcalendarTimeline.prototype.manageEndDateChange = function () {
  var startDate = new Date(this._formData.start_date);
  var startDateIsValid = (startDate.toString() != 'Invalid Date');

  var endDate = new Date(this._formData.end_date);
  var endDateIsValid = (endDate.toString() != 'Invalid Date');

  var startTimeZone = this._formData.resource_planner_event_dtstart_timezone;
  moment.tz.setDefault(startTimeZone);   // Set start Timezone
  var startDateTZ = moment(this._formData.start_date);

  var endTimeZone = this._formData.resource_planner_event_dtend_timezone;
  moment.tz.setDefault(endTimeZone);   // Set end Timezone
  var endDateTZ = moment(this._formData.end_date);

  moment.tz.setDefault(startTimeZone);  // Set start TZ to convert end time to start TZ and calculate duration
  endDateTZ = moment(endDateTZ).format(); // end date based on start TZ

  if (endDateIsValid && startDateIsValid) {
    if (moment(endDateTZ).format() <= moment(startDateTZ).format()) {
      //if (endDate.getTime() <= startDate.getTime()) {
      // End Date must be >= start Date
      // endDateTZ = startDateTZ;
      // endDate = startDate;
      this.getFormField(formField().DURATION).val('');
      this.getFormField(formField().DTEND).val('');
      //this.getFormField(formField().DTEND).val(formatDate(endDateTZ));
      //this.getFormField(formField().TZOFFSETTO).val(startTimeZone);
    } else {
      // Update duration
      this._formData.duration = moment(endDateTZ).diff(moment(startDateTZ), this._formData.duration_units);
      //this._formData.duration = endDate.diff(startDate, this._formData.duration_units);
      this.getFormField(formField().DURATION).val(this._formData.duration);
    }
  } else {
    if (!endDateIsValid) {
      this.getFormField(formField().DURATION).val('');
      this.getFormField(formField().DTEND).val('');
      this.getFormField(formField().ALLDAY)[0].checked = false;
    }
  }
}

//=============================================================================
// If duration_units changed, validate and update DURATION
FullcalendarTimeline.prototype.manageDurationUnitsChange = function () {
  if (this._formData.duration.length) {

    var duration;

    // if (this._formData.event_rrule == 'no_repeat') {
    //   var startDate = new Date(this._formData.start_date);
    //   var startDateIsValid = (startDate.toString() != 'Invalid Date');
    //   var endDate = new Date(this._formData.end_date);
    //   var endDateIsValid = (endDate.toString() != 'Invalid Date');

    //   if (endDateIsValid && startDateIsValid) {
    //     if (endDate.getTime() >= startDate.getTime()) {
    //       duration = moment.duration(endDate.diff(startDate, 'minute'), 'minute').as(this._formData.duration_units);
    //     }
    //   }
    // } else {
    duration = moment.duration(Number(this._formData.duration), this._formData.previous_duration_units).as(this._formData.duration_units);
    // }

    if (typeof duration !== 'undefined') {
      if (this._formData.duration_units.startsWith("minute")) {
        duration = Math.round(duration);
      }

      this._formData.duration = duration;
      this.getFormField(formField().DURATION).val(this._formData.duration);
    }
  }
}

//=============================================================================
// Handle recurrence selector (event_rrule) change
FullcalendarTimeline.prototype.handleRecurrenceSelectorChange = function () {
  this.getFormField('.recurring_event_data').hide(); //'slow');
  this.getFormField('.range_of_recurence').hide(); //'slow');

  var selected_recurrence = this._formData.event_rrule;
  this.clearFields(selected_recurrence);       // clear all fields of previous screens...

  if (selected_recurrence && selected_recurrence !== 'no_repeat') {
    this.getFormField('.recurring_event_data.' + selected_recurrence).show(); //'slow');
    this.getFormField('.range_of_recurence').show(); //'slow');

    //this.getFormField(formField().DTEND)[0].readOnly = true;
    // this.getFormField(formField().DTEND)[0].disabled = true;
    // this.getFormField(formField().TZOFFSETTO)[0].disabled = true;
  } else {
    //this.getFormField(formField().DTEND)[0].readOnly = false;
    // this.getFormField(formField().DTEND)[0].disabled = false;
    // this.getFormField(formField().TZOFFSETTO)[0].disabled = false;
  }
}

//=============================================================================

// Capture form fields -> 'window' data
FullcalendarTimeline.prototype.formUpdateHandler = function () {
  this.captureFormData();
  this.updateIcalData();
  //this.prepareEventsArray();
  //this.renderEvents();
}

//=============================================================================

/**
 * Capture form fields -> formData
 */
FullcalendarTimeline.prototype.captureFormData = function () {
  this._formData.title = $.trim(this.getFormField(formField().SUMMARY).val());
  this._formData.description = $.trim(this.getFormField(formField().DESCRIPTION).val());

  this._formData.all_day = this.getFormField(formField().ALLDAY)[0].checked;

  this._formData.start_date = $.trim(this.getFormField(formField().DTSTART).val());
  this._formData.resource_planner_event_dtstart_timezone = $.trim(this.getFormField(formField().TZOFFSETFROM).val());
  this._formData.start_date_with_timezone = moment.tz(this._formData.start_date, this._formData.resource_planner_event_dtstart_timezone).format();

  this._formData.duration = $.trim(this.getFormField(formField().DURATION).val());

  var duration_units = $.trim(this.getFormField(formField().DURATION_UNITS).find('option:selected').val()).toLowerCase();
  if (!isNaN(duration_units)) {
    // If val() is a number, take the text() instead
    duration_units = $.trim(this.getFormField(formField().DURATION_UNITS).find('option:selected').text()).toLowerCase();
  }

  if ((typeof this._formData.previous_duration_units === 'undefined' || this._formData.previous_duration_units != duration_units)
    && (typeof this._formData.duration_units !== 'undefined')) {
    this._formData.previous_duration_units = this._formData.duration_units;
  }
  this._formData.duration_units = duration_units;

  // recurrence rule - daily, weekly, monthly, etc.
  this._formData.event_rrule = $.trim(this.getFormField('#event_rrule').val()).toLowerCase();

  // no_repeat fields
  this._formData.end_date = $.trim(this.getFormField(formField().DTEND).val());
  this._formData.resource_planner_event_dtend_timezone = $.trim(this.getFormField(formField().TZOFFSETTO).val());
  this._formData.end_date_with_timezone = moment.tz(this._formData.end_date, this._formData.resource_planner_event_dtend_timezone).format();

  // daily fields
  this._formData.event_count_every_n_days = this.getFormField('#event_count_every_n_days')[0].checked;
  this._formData.event_count_every_weekday = this.getFormField('#event_count_every_weekday')[0].checked;
  this._formData.event_count = $.trim(this.getFormField('#event_count').val());

  // weekly fields
  // event_count (See above.... In the 'weekly' case, it always refers to 'every n weeks')
  var selected_weekdays = '';
  this.getFormField('.recurring_event_data.weekly .day_buttons input[type=checkbox]').each(function (index) {
    if ($(this)[0].checked) {
      var name = $(this)[0].name;
      var day_of_week = name.substring(name.lastIndexOf('[') + 1, name.lastIndexOf(']'));
      selected_weekdays = selected_weekdays + day_of_week + '/';
    }
  });
  this._formData.selected_weekdays = selected_weekdays.substring(0, selected_weekdays.length - 1);


  // monthly fields
  // 20.07.2020 this._formData.resource_planner_event_count_every_month = this.getFormField('#resource_planner_event_count_every_month')[0].checked;
  this._formData.event_count_every_month_by_day_of_the_week = this.getFormField('#event_count_every_month_by_day_of_the_week')[0].checked;

  // event_count (See above.... In the 'monthly' case, it always refers to 'every n months')
  this._formData.event_bymonthday = $.trim(this.getFormField('#event_bymonthday').val());
  this._formData.resource_planner_event_bymonthday_day = $.trim(this.getFormField('#resource_planner_event_bymonthday_day').val());
  this._formData.event_count = $.trim(this.getFormField('#event_count').val());
  // var every_n_months = this.getFormField('.recurring_event_data.monthly input#event_count').val();


  this._formData.event_count_every_month_day = this.getFormField('#event_count_every_month_day')[0].checked;
  var selected_days_of_month = '';
  this.getFormField('.recurring_event_data.monthly .monthly_day_buttons input[type=checkbox]').each(function (index) {
    if ($(this)[0].checked) {
      var name = $(this)[0].name;
      var date = name.substring(name.lastIndexOf('[') + 1, name.lastIndexOf(']'));
      selected_days_of_month = selected_days_of_month + date + '/';
    }
  });
  this._formData.selected_days_of_month = selected_days_of_month.substring(0, selected_days_of_month.length - 1);


  // range_of_recurence fields
  this._formData.event_until_no_end_date = this.getFormField('#event_until_no_end_date')[0].checked;
  this._formData.event_until_create_n_events = this.getFormField('#event_until_create_n_events')[0].checked;
  this._formData.event_until_count = $.trim(this.getFormField('#event_until_count').val());
  this._formData.event_until_repeat_until = this.getFormField('#event_until_repeat_until')[0].checked;
  this._formData.event_repeat_until_date = $.trim(this.getFormField('#event_repeat_until_date').val());
}

//=============================================================================

/**
 * Convert formData into iCalData
 */
FullcalendarTimeline.prototype.updateIcalData = function () {
  // This could be performed on the server-side
}

//=============================================================================
// If end_date changed validate and update DURATION

FullcalendarTimeline.prototype.validateFormData = function () {

  this.captureFormData();

  var startDate = new Date(this._formData.start_date);
  var startDateIsValid = (startDate.toString() != 'Invalid Date');
  var endDate = new Date(this._formData.end_date);
  var endDateIsValid = (endDate.toString() != 'Invalid Date');
  var duration = this._formData.duration;
  //var durationUnits = this._formData.duration_units;

  // Title
  if (this._formData.title) {
    this.getFormField(formField().SUMMARY).removeClass('input_warning');
  } else {
    this.getFormField(formField().SUMMARY).addClass('input_warning');
  }

  // Start/End/Duration
  if (startDateIsValid) {
    this.getFormField(formField().DTSTART).removeClass('input_warning');
  } else {
    startDate = '';
    startDateIsValid = false;
    this.getFormField(formField().DTSTART).addClass('input_warning');
    this.getFormField(formField().DTSTART).val('');
    endDate = '';
    endDateIsValid = false;
    this.getFormField(formField().DTEND).val('');
    this.getFormField(formField().DURATION).val('');
    duration = '';
    this.getFormField(formField().ALLDAY)[0].checked = false;
  }

  if (duration && !isNaN(duration)) {
    this.getFormField(formField().DURATION).removeClass('input_warning');
  } else {
    this.getFormField(formField().DURATION).val('');
    duration = '';
    this.getFormField(formField().DURATION).addClass('input_warning');
  }

  if (endDateIsValid) {
    this.getFormField(formField().DTEND).removeClass('input_warning');
  } else {
    this.getFormField(formField().DTEND).addClass('input_warning');
  }

  // Reccurence pattern
  switch (this._formData.event_rrule) {
    case 'no_repeat':
      // NOTHING TO DO
      break;

    case 'hourly':
      // NOT IMPLEMENTED
      break;

    case 'daily':
      // var startTimeZone = this._formData.resource_planner_event_dtstart_timezone;
      // this.getFormField(formField().TZOFFSETTO).val(startTimeZone);
      // 05-09-2020 this.getFormField(formField().DTEND).removeClass('input_warning');

      var every_n_days_checkbutton = this.getFormField('.recurring_event_data.daily input#event_count_every_n_days')[0];
      var every_weekday_checkbutton = this.getFormField('.recurring_event_data.daily input#event_count_every_weekday')[0];

      if (every_n_days_checkbutton.checked) {
        var event_count_value = this.getFormField('.recurring_event_data.daily input#event_count').val();
        if (event_count_value.length && !isNaN(event_count_value)) {
          this.getFormField('.recurring_event_data.daily').removeClass('input_warning');
        } else {
          this.getFormField('.recurring_event_data.daily').addClass('input_warning');
        }
      } else if (every_weekday_checkbutton.checked) {
        this.getFormField('.recurring_event_data.daily').removeClass('input_warning');
      } else {
        this.getFormField('.recurring_event_data.daily').addClass('input_warning');
      }
      break;

    case 'weekly':
      // startTimeZone = this._formData.resource_planner_event_dtstart_timezone;
      // this.getFormField(formField().TZOFFSETTO).val(startTimeZone);

      // 05-09-2020 this.getFormField(formField().DTEND).removeClass('input_warning');

      var every_n_weeks = this.getFormField('.recurring_event_data.weekly input');

      var every_n_weeks_value = parseInt(every_n_weeks.val());
      if (isNaN(every_n_weeks_value)) {
        this.getFormField('.recurring_event_data.weekly').addClass('input_warning');
      } else {
        var selectedWeekDays = this.getFormField('.recurring_event_data.weekly label.day-button-selected');
        if (!selectedWeekDays.length) {
          this.getFormField('.recurring_event_data.weekly').addClass('input_warning');
        } else {
          this.getFormField('.recurring_event_data.weekly').removeClass('input_warning');
        }
      }
      break;

    case 'bi_weekly':
      // NOT IMPLEMENTED
      // 05-09-2020 this.getFormField(formField().DTEND).removeClass('input_warning');
      break;

    case 'monthly':
      // startTimeZone = this._formData.resource_planner_event_dtstart_timezone;
      // this.getFormField(formField().TZOFFSETTO).val(startTimeZone);

      // 05-09-2020 this.getFormField(formField().DTEND).removeClass('input_warning');

      var every_month_by_day_of_the_week = this.getFormField('.recurring_event_data.monthly input#event_count_every_month_by_day_of_the_week')[0];
      var every_month_day_checkbutton = this.getFormField('.recurring_event_data.monthly input#event_count_every_month_day')[0];

      //if (every_n_months_checkbutton.checked) {
      var every_n_months_input = this.getFormField('.recurring_event_data.monthly input#event_count');
      var every_n_months_value = parseInt(every_n_months_input.val());
      if (isNaN(every_n_months_value) || every_n_months_input.val() <= 0) {
        this.getFormField('.recurring_event_data.monthly').addClass('input_warning');
        this.getFormField('.recurring_event_data.monthly input#event_count').val('');
      } else if (every_month_by_day_of_the_week.checked) {
        this.getFormField('.recurring_event_data.monthly').removeClass('input_warning');
      } else if (every_month_day_checkbutton.checked) {
        var selectedMonthDays = this.getFormField('.recurring_event_data.monthly label.day-button-selected');
        if (!selectedMonthDays.length) {
          this.getFormField('.recurring_event_data.monthly').addClass('input_warning');
        } else {
          this.getFormField('.recurring_event_data.monthly').removeClass('input_warning');
        }
      } else {
        this.getFormField('.recurring_event_data.monthly').addClass('input_warning');
      }
      break;

    case 'annually':
      // NOT IMPLEMENTED
      break;

    case 'freeform':
      // NOT IMPLEMENTED
      break;

    default:
      break;
  }

  // Range of recurrence
  var until_no_end_date_checkbutton = this.getFormField('.range_of_recurence input#event_until_no_end_date')[0];
  var until_create_n_events_checkutton = this.getFormField('.range_of_recurence input#event_until_create_n_events')[0];
  var repeat_until_checkbutton = this.getFormField('.range_of_recurence input#event_until_repeat_until')[0];

  if (until_no_end_date_checkbutton.checked) {
    this.getFormField('.range_of_recurence').removeClass('input_warning');
    this.getFormField('.range_of_recurence input#event_repeat_until_date').val('');
    this.getFormField('.range_of_recurence input#event_until_count').val('');
  } else if (until_create_n_events_checkutton.checked) {
    var create_n_events = this.getFormField('.range_of_recurence input#event_until_count').val();
    var create_n_events_value = parseInt(create_n_events);
    if (isNaN(create_n_events_value)) {
      this.getFormField('.range_of_recurence').addClass('input_warning');
    } else {
      this.getFormField('.range_of_recurence').removeClass('input_warning');
      this.getFormField('.range_of_recurence input#event_repeat_until_date').val('');
    }
  } else if (repeat_until_checkbutton.checked) {
    var untilDate = this.getFormField('.range_of_recurence input#event_repeat_until_date')[0];
    var validDate = Date.parse(untilDate.value);
    this.getFormField('#event_until_count').val('');
    if (validDate) {
      var start_date = new Date(moment(this._formData.start_date).startOf('day'));
      var newUntilDate = new Date(untilDate.value);
      if (newUntilDate < start_date) {
        untilDate.value = '';
        this.getFormField('.range_of_recurence').addClass('input_warning');
      } else {
        this.getFormField('.range_of_recurence').removeClass('input_warning');
      }
    } else {
      this.getFormField('.range_of_recurence').addClass('input_warning');
      this.getFormField('.range_of_recurence input#event_until_repeat_until')[0].checked = false;
    }
  } else {
    this.getFormField('.range_of_recurence').addClass('input_warning');
  }

}

//=============================================================================
// clear all fields

FullcalendarTimeline.prototype.clearFields = function (selected_recurrence) {
  // DTSTART / DURATION / DTEND
  // $.trim(this.getFormField(formField().DTSTART).val(''));
  // 05-09-2020 $.trim(this.getFormField(formField().DURATION).val(''));
  // 05-09-2020 $.trim(this.getFormField(formField().DTEND).val(''));
  this.getFormField(formField().ALLDAY)[0].checked = false;

  // Clear DAILY Fields
  var buttonChecked = this.getFormField('.recurring_event_data.daily input#event_count_every_n_days');
  buttonChecked[0].checked = false;
  this.getFormField('.recurring_event_data.daily input#event_count').val('');
  buttonChecked = this.getFormField('.recurring_event_data.daily input#event_count_every_weekday');
  buttonChecked[0].checked = false;

  // Clear WEEKLY Fields
  this.getFormField('.recurring_event_data.weekly input').val('');
  this.getFormField('.recurring_event_data.weekly .day_buttons input[type=checkbox]').each(function (index) {
    $(this)[0].checked = false;
  });
  this.getFormField('.day-button-selected').removeClass('day-button-selected');

  // Clear MONTHLY Fields
  //buttonChecked = this.getFormField('.recurring_event_data.monthly input#resource_planner_event_count_every_month');
  //buttonChecked[0].checked = false;
  this.getFormField('.recurring_event_data.monthly input#event_count').val('');
  var selectedCheckedButton = this.getFormField('.recurring_event_data.monthly input#event_count_every_month_day');
  selectedCheckedButton[0].checked = false;
  this.getFormField('.recurring_event_data.monthly input#event_count_every_month_by_day_of_the_week')[0].checked = false;

  this.getFormField('.recurring_event_data.monthly .monthly_day_buttons input[type=checkbox]').each(function (index) {
    $(this)[0].checked = false;
  });
  var selectedMonthDays = this.getFormField('.recurring_event_data.monthly label.day-button-selected');
  selectedMonthDays.removeClass('day-button-selected');

  // Clear range_of_recurence
  this.getFormField('#event_until_no_end_date')[0].checked = false;
  this.getFormField('#event_until_create_n_events')[0].checked = false;
  this.getFormField('#event_until_count').val('');
  this.getFormField('.range_of_recurence input#event_until_repeat_until')[0].checked = false;
  this.getFormField('.range_of_recurence input#event_repeat_until_date').val('');

} // clear all fields

//=============================================================================
// all_day checkbox handler
//=============================================================================
FullcalendarTimeline.prototype.allDayHandler = function () {

  var startTimeZone = this._formData.resource_planner_event_dtstart_timezone;
  var startDate = this._formData.start_date
  startDate = moment.tz(startDate, startTimeZone);

  var all_day = this._formData.all_day;
  if (all_day) {              // all_day checkbox is checked
    if (moment(startDate).format() == "Invalid date") {
      startDate = moment();
      startDate = moment.tz(startDate, startTimeZone);
    }

    moment.tz.setDefault(startTimeZone);
    startDate = moment(startDate).startOf('day').format();  // set start date to midnight
    var endDate = moment(startDate).add(1, 'days');         // set end date to the next start date day
    var duration = moment(endDate).diff(startDate, this._formData.duration_units); // calculate duration

    var endTimeZone = this._formData.resource_planner_event_dtend_timezone; // end date timezone
    endDate = moment.tz(endDate, endTimeZone);

    this.getFormField(formField().DTSTART).val(moment(startDate).format('YYYY-MM-DD HH:mm'));
    this.getFormField(formField().DTEND).val(moment(endDate).format('YYYY-MM-DD HH:mm'));
    this.getFormField(formField().DURATION).val(duration);
    this.getFormField(formField().ALLDAY)[0].checked = true;
  }
}   // end of all_day checkbox handler
//=============================================================================

FullcalendarTimeline.prototype.updateEventUnitsFormFields = function () {

  if (typeof this._eventUnits !== 'undefined' && this._eventUnits.length > 0) {

    // Update EventsArray session cookie
    var cookieName = this._formFieldsId + '_eventUnits';
    var eventUnitsJSON = JSON.stringify(this._eventUnits);
    setCookie(cookieName, eventUnitsJSON, 1);

    // Re-build and populate EventUnit fields on form

    // row template
    var row_template = [
      '<div class="row form-group">',
      '<div class="col">',

      '<input value="false" name="event[event_units_attributes][#0][_destroy]" id="event_event_units_attributes_#0__destroy" type="hidden">',

      '<label for="event_event_units_attributes_#0_start_at">Start at</label>',
      '<input value="#1" name="event[event_units_attributes][#0][start_at]" id="event_event_units_attributes_#0_start_at" readonly="readonly" type="text">',

      '<label for="event_event_units_attributes_#0_end_at">End at</label>',
      '<input value="#2" name="event[event_units_attributes][#0][end_at]" id="event_event_units_attributes_#0_end_at" readonly="readonly" type="text">',

      '<label for="event_event_units_attributes_#0_all_day">All day</label>',
      '<input name="event[event_units_attributes][#0][all_day]" type="hidden" value="#3">',
      '<input readonly="readonly" onclick="return false;" type="checkbox" value="#3" name="event[event_units_attributes][#0][all_day]" id="event_event_units_attributes_#0_all_day">',

      '<input name="event[event_units_attributes][#0][is_modified]" type="hidden" value="#3">',
      '<input readonly="readonly" onclick="return false;" type="checkbox" value="#3" name="event[event_units_attributes][#0][is_modified]" id="event_event_units_attributes_#0_modified">',

      '</div>',
      '</div>',
    ].join("\n");

    var eventUnitsFields = this.getFormField('#formEventUnitsFields');

    eventUnitsFields.empty(); // remove all child nodes

    // Add form data
    for (var eventUnitsIndex = 0; eventUnitsIndex < this._eventUnits.length; eventUnitsIndex++) {
      if (
        typeof this._eventUnits[eventUnitsIndex].startDate !== 'undefined' &&
        typeof this._eventUnits[eventUnitsIndex].endDate !== 'undefined'
      ) {
        var row = row_template.
          replace(/#0/g, eventUnitsIndex).
          replace(/#1/g, new Date(this._eventUnits[eventUnitsIndex].startDate).toISOString()).
          replace(/#2/g, new Date(this._eventUnits[eventUnitsIndex].endDate).toISOString()).
          replace(/#3/g, '0');

        eventUnitsFields.append(row);
      }
    }
  }
}

// GUI Timezone Handler (FROM)
//=============================================================================
FullcalendarTimeline.prototype.timeZoneConverter = function () {

  var startTimeZone = this._formData.resource_planner_event_dtstart_timezone;
  var endTimeZone = this._formData.resource_planner_event_dtend_timezone;

  var duration = $.trim(this.getFormField(formField().DURATION).val());
  var durationUnits = this._formData.duration_units.toLowerCase();
  if (duration == '') {
    this.getFormField(formField().DTEND).val('');
    return;
  }

  var startDate = this._formData.start_date;
  startDate = moment.tz(startDate, startTimeZone);
  var endDate = dateAdd(startDate, duration, durationUnits);   // Calculate end date
  endDate = moment.tz(endDate, endTimeZone);
  this.getFormField(formField().DTEND).val(moment(endDate).format());

  endDate = dateAdd(startDate, duration, durationUnits);   // Calculate end date
  endDate = moment.tz(endDate, endTimeZone);

  moment.tz.setDefault(endTimeZone);   // Set end Timezone
  this.getFormField(formField().DTEND).val(moment(endDate).format("YYYY-MM-DD HH:mm"));

  return;
}
