require_dependency 'resource_planner/application_controller'

module ResourcePlanner
  class EventsController < ApplicationController
    before_action :set_event, only: [:show, :edit, :update, :destroy]

    # GET /events
    def index
      @events = ::ResourcePlanner::Event.events

      respond_to do |format|
        format.html # index.html.erb
        # format.html { render ('index.' + ( params[:view].nil? ? '' : "#{params[:view]}." ) + 'html.erb') }
        format.xml  { render :xml => @events }
        format.json # index.json.jbuilder
        # format.json { render ('index.json.jbuilder') }
        # format.json { render :json => @events }
      end
    end

    # GET /events/1
    def show
      respond_to do |format|
        format.html # show.html.erb
        # format.html { render ('show.' + ( params[:view].nil? ? '' : "#{params[:view]}." ) + 'html.erb') }
        format.xml  { render :xml => @event }
        format.json { render :json => @event }
      end
    end

    # GET /events/new
    def new
      @view = params[:view]
      @event = ::ResourcePlanner::Event.new
      @event.event_type = ::ResourcePlanner::EventType.event
    end

    # GET /events/1/edit
    def edit
    end

    # POST /events
    def create
      # if params['formData'] && params['resources'] && params['eventUnits']
      #   # This is from the ajax call

      #   # Verify that all data is present
      #   # TODO: Improve the data validation
      #   if params['formData'].empty? || params['resources'].empty? || params['eventUnits'].empty?
      #     respond_to do |format|
      #       format.html { render :new }
      #       format.xml  { render :xml => { :error => 'Incomplete data' }, :status => :unprocessable_entity }
      #       format.json { render :json => { :error => 'Incomplete data' }, :status => :unprocessable_entity }
      #     end
      #   end

      #   # TODO: Add some validation here (SQL injection?)

      #   #start_at = Time.parse( params['formData']['start_date'] )
      #   dtstart_timezone = params['formData']['resource_planner_event_dtstart_timezone']
      #   start_at = ActiveSupport::TimeZone[dtstart_timezone].parse( params['formData']['start_date'] )
      #   start_at = Time.parse( start_at.to_s )
      #   end_at = start_at + (60 * params['eventUnits'][0]['eventDuration'].to_i)
      #   end_at = Time.parse( end_at.to_s )

      #   @event = ::ResourcePlanner::Event.new(
      #     {
      #       :event_type => ::ResourcePlanner::EventType.event,
      #       :name => params['formData']['title'],
      #       :description => params['formData']['description'],
      #       :start_at => start_at,
      #       :end_at => end_at,
      #       :ical_expression => '',
      #       :all_day => false,
      #     } #,:without_protection => true
      #   )

      #   params['resources'].each do |resource_string|
      #     object_name = resource_string.split('-')[0]
      #     object_id = (resource_string.split('-')[1]).to_i
      #     object = (object_name.constantize).find(object_id)
      #     @event.subscriptions.new(:resource => object)
      #     #@event.subscriptions << object
      #   end

      # else
      @event = ::ResourcePlanner::Event.new(event_params)
      # end

      respond_to do |format|
        if @event.save
          format.html { redirect_to @event, notice: 'Event was successfully created.' }
          format.xml  { render :xml => @event, :status => :created, :location => @event }
          format.json { render :json => @event, :status => :created, :location => @event }
          # format.xml  { head :ok }
          # format.json { head :no_content }
        else
          format.html { render :new }
          format.xml  { render :xml => @event.errors, :status => :unprocessable_entity }
          format.json { render :json => @event.errors, :status => :unprocessable_entity }
        end
      end
    end

    # PATCH/PUT /events/1
    def update
      if @event.update(event_params)
        format.html { redirect_to @event, notice: 'Event was successfully updated.' }
        format.xml  { head :ok }
        format.json { head :no_content }
      else
        format.html { render :edit }
        format.xml  { render :xml => @event.errors, :status => :unprocessable_entity }
        format.json { render :json => @event.errors, :status => :unprocessable_entity }
      end
    end

    # DELETE /events/1
    def destroy
      @event.destroy

      respond_to do |format|
        format.html { redirect_to events_url, notice: 'Event was successfully destroyed.' }
        format.xml  { head :ok }
        format.json { head :no_content }
      end
    end

    private

    # Use callbacks to share common setup or constraints between actions.
    def set_event
      @event = ::ResourcePlanner::Event.find(params[:id])
    end

    # Only allow a trusted parameter "white list" through.
    def event_params
      # params.fetch(:event, {})

      params.require(:event).except(
        # Unwanted parameters
        :start_at, :end_at, :all_day,
        :duration, :duration_units
      ).permit(
        # Permitted parameters
        :event_owner, :event_owner_id, :event_owner_type,
        :parent_event, :parent_event_id,
        :event_type, :event_type_id, :event_type_type, :event_type_gid,
        :name, :description,
        :dtstart_timezone,
        :dtend_timezone,

        :rrule,
        :count,

        :day_of_week,
        :day_of_month,

        :byday,
        :bymonthday,
        :bymonthday_day,

        :until,
        :until_count,
        :repeat_until_date,

        :ical_expression,
        :price, :currency, :currency_id,

        event_units_attributes: [
          :_destroy, :id,
          :start_at, :end_at,
          :all_day,
          :is_modified,

          subscriptions_attributes: [
            :_destroy, :id,
            :event_unit, :event_unit_id,
            :resource, :resource_id, :resource_type, 
            :ambit,
            :quantity,
            :name,
            :notes
          ]
        ]
      )
    end
  end
end
