# frozen_string_literal: true

require 'ice_cube'

# TODO: Take advantage of the ice_cube gem
#       ::IceCube::Rule.weekly.day(:friday).day_of_month(13).month_of_year(:october).to_yaml ?

module ResourcePlanner
  class Event < ApplicationRecord
    # attr_accessible :parent_event, :parent_event_id
    # attr_accessible :name, :description, :start_at, :end_at, :all_day
    # attr_accessible :ical_expression

    # data required for creating iCal definition
    # RFC 5545                       iCalendar                  September 2009
    #
    # DTSTART;TZID=America/New_York:19980119T020000
    # DTEND;TZID=America/New_York:19980119T030000
    #
    # The "TZID" property parameter MUST NOT be applied to DATE
    # properties and DATE-TIME or TIME properties whose time values are
    # specified in UTC.
    #
    # attr_accessor :dtstart
    # attr_accessor :dtend
    # attr_accessor :timezone
    #
    # attr_accessor :duration
    #
    # Recurrence rules
    # recur           = recur-rule-part *( ";" recur-rule-part )
    # ;
    # ; The rule parts are not ordered in any
    # ; particular sequence.
    # ;
    # ; The FREQ rule part is REQUIRED,
    # ; but MUST NOT occur more than once.
    # ;
    # ; The UNTIL or COUNT rule parts are OPTIONAL,
    # ; but they MUST NOT occur in the same 'recur'.
    # ;
    # ; The other rule parts are OPTIONAL,
    # ; but MUST NOT occur more than once.
    #
    # attr_accessor :exdate
    # attr_accessor :rdate
    # attr_accessor :rrule
    #
    # attr_accessor :until
    # attr_accessor :count
    # attr_accessor :interval
    #
    # recur-rule-part = ( "FREQ" "=" freq )
    #     / ( "UNTIL" "=" enddate )
    #     / ( "COUNT" "=" 1*DIGIT )
    #     / ( "INTERVAL" "=" 1*DIGIT )
    #     / ( "BYSECOND" "=" byseclist )
    #     / ( "BYMINUTE" "=" byminlist )
    #     / ( "BYHOUR" "=" byhrlist )
    #     / ( "BYDAY" "=" bywdaylist )
    #     / ( "BYMONTHDAY" "=" bymodaylist )
    #     / ( "BYYEARDAY" "=" byyrdaylist )
    #     / ( "BYWEEKNO" "=" bywknolist )
    #     / ( "BYMONTH" "=" bymolist )
    #     / ( "BYSETPOS" "=" bysplist )
    #     / ( "WKST" "=" weekday )
    #
    # freq        = "SECONDLY" / "MINUTELY" / "HOURLY" / "DAILY"
    # / "WEEKLY" / "MONTHLY" / "YEARLY"
    #
    # enddate     = date / date-time
    #
    # byseclist   = ( seconds *("," seconds) )
    #
    # seconds     = 1*2DIGIT       ;0 to 60
    #
    # byminlist   = ( minutes *("," minutes) )
    #
    # minutes     = 1*2DIGIT       ;0 to 59
    #
    # byhrlist    = ( hour *("," hour) )
    #
    # hour        = 1*2DIGIT       ;0 to 23
    #
    # bywdaylist  = ( weekdaynum *("," weekdaynum) )
    #
    # weekdaynum  = [[plus / minus] ordwk] weekday
    #
    # plus        = "+"
    #
    # minus       = "-"
    #
    # ordwk       = 1*2DIGIT       ;1 to 53
    #
    # weekday     = "SU" / "MO" / "TU" / "WE" / "TH" / "FR" / "SA"
    # ;Corresponding to SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY,
    # ;FRIDAY, and SATURDAY days of the week.
    #
    # bymodaylist = ( monthdaynum *("," monthdaynum) )
    #
    # monthdaynum = [plus / minus] ordmoday
    #
    # ordmoday    = 1*2DIGIT       ;1 to 31
    #
    # byyrdaylist = ( yeardaynum *("," yeardaynum) )
    #
    # yeardaynum  = [plus / minus] ordyrday
    #
    # ordyrday    = 1*3DIGIT      ;1 to 366
    #
    # bywknolist  = ( weeknum *("," weeknum) )
    #
    # weeknum     = [plus / minus] ordwk
    #
    # bymolist    = ( monthnum *("," monthnum) )
    #
    # monthnum    = 1*2DIGIT       ;1 to 12
    #
    # bysplist    = ( setposday *("," setposday) )
    #
    # setposday   = yeardaynum    attr_accessor :until
    #
    # attr_accessor :bysecond
    # attr_accessor :byminute
    # attr_accessor :byhour
    # attr_accessor :byday
    # attr_accessor :bymonthday
    # attr_accessor :byyearday
    # attr_accessor :byweekno
    # attr_accessor :bymonth
    # attr_accessor :bysetpos
    # attr_accessor :wkst
    # attr_accessor :freq

    #--------------------------------------------------------------------------

    # Virtual attributes
    # ------------------
    attr_accessor :duration_units_tmp, :all_day_tmp

    def start_at=(value)
      self.dtstart_datetime = value
      self.dtstart = value.is_a?(String) ? value : value.strftime('%Y-%m-%d %H:%M') # Time.new(value.to_s).strftime('%Y-%m-%d %H:%M')

      unless event_units.empty?
        event_unit = event_units.order(:start_at).first
        event_unit = event_units.first if event_unit.nil?
        event_unit.start_at = value
      end
    end

    def start_at
      unless event_units.empty?
        event_unit = event_units.order(:start_at).first
        event_unit = event_units.first if event_unit.nil?
        self.dtstart_datetime = event_unit.start_at
      end

      self.dtstart_datetime
    end

    def start_at_timezone=(value)
      self.dtstart_timezone = value
    end

    def start_at_timezone
      self.dtstart_timezone
    end

    def end_at=(value)
      self.dtend_datetime = value
      self.dtend = value.is_a?(String) ? value : value.strftime('%Y-%m-%d %H:%M') # Time.new(value.to_s).strftime('%Y-%m-%d %H:%M')

      unless event_units.empty?
        event_unit = event_units.order(:end_at).last
        event_unit = event_units.last if event_unit.nil?
        event_unit.end_at = value
      end
    end

    def end_at
      unless event_units.empty?
        event_unit = event_units.order(:end_at).last
        event_unit = event_units.last if event_unit.nil?
        self.dtend_datetime = event_unit.end_at
      end

      self.dtend_datetime
    end

    def end_at_timezone=(value)
      self.dtend_timezone = value
    end

    def end_at_timezone
      self.dtend_timezone
    end

    def units=(value)
      @duration_units_tmp = value
    end

    def units
      @duration_units_tmp
    end

    def all_day=(value)
      if event_units.size == 1
        event_units.first.all_day = value
      else
        @all_day_tmp = value
      end
    end

    def all_day
      if event_units.size == 1
        event_units.first.all_day
      else
        @all_day_tmp
      end
    end

    #--------------------------------------------------------------------------

    #enum :event_type => [:event, :availability, :restriction]

    #has_event_calendar

    belongs_to :event_owner, polymorphic: true, optional: true

    belongs_to :parent_event, class_name: '::ResourcePlanner::Event', dependent: :destroy, optional: true

    belongs_to :event_type, polymorphic: true

    belongs_to :currency, :class_name => '::CoreModels::Base::Currency', optional: true

    has_many   :sub_events, inverse_of: :parent_event, class_name: '::ResourcePlanner::Event', foreign_key: 'parent_event_id'
    accepts_nested_attributes_for :sub_events, allow_destroy: true,
                                               reject_if: ->(attributes) { attributes['name'].nil? }
  
    has_many :event_units, class_name: '::ResourcePlanner::EventUnit', foreign_key: 'event_id', autosave: true, dependent: :destroy
    accepts_nested_attributes_for :event_units, allow_destroy: true

    has_many :recurring_event_exceptions, class_name: '::ResourcePlanner::Event', foreign_key: 'parent_event_id', dependent: :destroy
    accepts_nested_attributes_for :recurring_event_exceptions, allow_destroy: true

    #----------------------------------------------------------------------------

    # Handlers for polymorphic values (using Global ID)

    # Use :to_global_id to populate forms
    def event_type_gid
      event_type&.to_global_id
    end

    # Set the attribute from a Global ID (handles the form submission)
    def event_type_gid=(gid)
      self.event_type = GlobalID::Locator.locate gid
    end

    # Use :to_global_id to populate forms
    def event_owner_gid
      event_owner&.to_global_id
    end

    # Set the attribute from a Global ID (handles the form submission)
    def event_owner_gid=(gid)
      self.event_owner = GlobalID::Locator.locate gid
    end

    #--------------------------------------------------------------------------

    #---------- Validations

    # validates  :name, presence: true, uniqueness: true

    #validates :event_units, :presence => true
    #validates :event_units, :size => { :minimum => 1 }
    #validates :event_units, :length => { :minimum => 1 }

    validates_associated :sub_events

    #---------- Scopes

    scope :events, -> { where(:event_type => ::ResourcePlanner::EventType.event) }
    scope :resource_availabilities, -> { where(:event_type => ::ResourcePlanner::EventType.availability) }
    scope :resource_restrictions, -> { where(:event_type => ::ResourcePlanner::EventType.restriction) }

    #---------- Callbacks

    after_initialize  :ensure_at_least_one_event_unit_exists
    before_validation :associate_and_update_event_units
    before_validation :set_ical_expression

    #--------------------------------------------------------------------------

    # TBD: Remove this when EventCalendar is fixed
    # Get the events overlapping the given start and end dates
    # def self.events_for_date_range(start_d, end_d, find_options = {})
    #   self.where(find_options)
    #   .where("(? <= #{self.quoted_table_name}.#{self.end_at_field}) AND (#{self.quoted_table_name}.#{self.start_at_field}< ?)", start_d.to_time.utc, end_d.to_time.utc)
    #   .order("#{self.quoted_table_name}.#{self.start_at_field} ASC")
    # end

    #--------------------------------------------------------------------------

    def full_details
      ::IceCube::Rule.from_yaml(ical_expression)
    end

    def all_day?
      all_day
    end

    #--------------------------------------------------------------------------

    # are there any active conflicts
    def has_conflicts?
      conflicts_count > 0
    end

    # return count of active conflicts
    def conflicts_count
      conflicts.length
    end

    # return active conflicts
    def conflicts
      active_conflicts = []

      event_units.each do |event_unit|
        event_unit.resources.each do |event_unit_resource|
          # quantity consumed by this event unit
          event_unit_subscription = event_unit.subscriptions.where(resource: event_unit_resource).first
          total_resource_consumed = event_unit_subscription&.quantity ? event_unit_subscription.quantity : 0

          coincident_event_units = event_unit_resource.event_units
                                                      .where.not(event_id: id)
                                                      .where(start_at: event_unit.start_at..event_unit.end_at)
                                                      .where(end_at: event_unit.start_at..event_unit.end_at)

          # coincident_event_units = ::ResourcePlanner::EventUnit
          #                          .where.not(event_id: id)
          #                          .where(start_at: event_unit.start_at..event_unit.end_at)
          #                          .where(end_at: event_unit.start_at..event_unit.end_at)

          potential_conflicts = [event_unit]

          coincident_event_units.each do |coincident_event_unit|
            coincident_event_unit.subscriptions.each do |coincident_event_unit_subscription|
              potential_conflicts.push(coincident_event_unit)
              total_resource_consumed += coincident_event_unit_subscription.quantity if coincident_event_unit_subscription.quantity
            end
          end

          active_conflicts.push(potential_conflicts) if event_unit_resource.quantity && total_resource_consumed > event_unit_resource.quantity
        end
      end

      active_conflicts
    end

    #--------------------------------------------------------------------------

    def to_s
      "#{name} (#{start_at}-#{end_at})"
    end

    #--------------------------------------------------------------------------

    private

    def to_ical_expression(count = 1)
      case repeating_name
      when 'secondly'
        ::IceCube::Rule.secondly(count).to_yaml
      when 'minutely'
        ::IceCube::Rule.minutely(count).to_yaml
      when 'hourly'
        ::IceCube::Rule.hourly(count).to_yaml
      when 'daily'
        ::IceCube::Rule.daily(count).to_yaml
      when 'weekly'
        ::IceCube::Rule.weekly(count).to_yaml
      when 'monthly'
        ::IceCube::Rule.monthly(count).to_yaml
      when 'annually'
        ::IceCube::Rule.yearly(count).to_yaml
      else
        ''
      end
    end

    #---------- Callback handlers

    # Ensure that there is at least one EventUnit
    def ensure_at_least_one_event_unit_exists
      if new_record? && event_units.empty?
        event_units.build
        event_units.first.attributes = {
          event: self
        }

        # could be:
        #
        # event_units.build(
        #   start_at: self.dtstart_datetime,
        #   end_at: self.dtend_datetime,
        #   all_day: all_day_tmp
        # )
        #
        # OR
        #
        # event_units.build([ {}, {}, {}, ... ])

      end
    end

    def associate_and_update_event_units
      event_units.each { |event_unit| event_unit.event = self }

      if event_units.length == 1 && !(self.dtstart_datetime.nil? || self.dtend_datetime.nil?)
        event_units.first.attributes = {
          name: self.name,
          start_at: self.dtstart_datetime,
          end_at: self.dtend_datetime,
          all_day: all_day_tmp
        }
      end
    end

    def set_ical_expression
      self.ical_expression = ::IceCube::Rule.yearly.day_of_month(23).day(:friday).month_of_year(:october).to_yaml
    end
  end
end
